using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class AttitudeRateController: PartModule {

	static System.Random rng = new System.Random();
	static float k1 = -0.125f; // Commanded AV at full input range
	static float k2 = 10.0f; // Control loop gain
	
	[KSPField(isPersistant = true)]
	bool isActive = true;
	bool debug = false;

	public override void OnStart(PartModule.StartState state) {
		print("AttitudeRateController.OnStart");
		if (vessel != null) {
			vessel.OnFlyByWire += flyByWire;
			RenderingManager.AddToPostDrawQueue(3, OnGui);
		}
	}
	
	public override void OnUpdate() {
		//print("AttitudeRateController.OnUpdate");
		status = isActive ? "Enabled" : "Disabled";
	}
	
//	public override void OnFixedUpdate() {
//		print("AttitudeRateController.OnFixedUpdate");
//	}
	
	void flyByWire(FlightCtrlState s) {
		//print("AttitudeRateController.flyByWire");
		Vector3d a = (Vector3d)vessel.transform.InverseTransformDirection(vessel.rigidbody.angularVelocity);
		float pitch = control((float)a.x, s.pitch);
		float roll = control((float)a.y, s.roll);
		float yaw = control((float)a.z, s.yaw);
		if (debug) {
			print(String.Format("     {0,8} {1,8} {2,8}", "Pitch", "Roll", "Yaw"));
			print(String.Format("AV:  {0,8:f3} {1,8:f3} {2,8:f3}", a.x, a.y, a.z));
			print(String.Format("IN:  {0,8:f3} {1,8:f3} {2,8:f3}", s.pitch, s.roll, s.yaw));
			print(String.Format("OUT: {0,8:f3} {1,8:f3} {2,8:f3}", pitch, roll, yaw));
		}
		if (isActive) {
			s.pitch = pitch;
			s.roll = roll;
			s.yaw = yaw;
		}
	}
	
	float control(float rate, float input) {
		float cmd = k1 * input;
		return Mathf.Clamp((rate - cmd) * k2, -1.0f, 1.0f);
	}
	
	//--------------------------------------------------------------------------------
	
	[KSPField(guiName = "Status", guiActive = true)]
	public string status;
	
	[KSPAction("Enable")]
	public void EnableAction(KSPActionParam param) {
		isActive = true;
	}
	
	[KSPAction("Disable")]
	public void DisableAction(KSPActionParam param) {
		isActive = false;
	}
	
	[KSPAction("Toggle")]
	public void ToggleAction(KSPActionParam param) {
		Toggle();
	}
	
	[KSPEvent(guiName = "Toggle", guiActiveUnfocused = true, externalToEVAOnly = true, guiActive = true, unfocusedRange = 2f)]
	public void Toggle() {
		isActive = !isActive;
	}

	//--------------------------------------------------------------------------------
	
	static int winID = rng.Next();
	static Rect winDrag = new Rect(0, 0, 10000, 20);
	static Rect defaultWinRect = new Rect(Screen.width - 50, Screen.height - 200, 50, 50);
	
	[KSPField(isPersistant = true)]
	Rect winRect = defaultWinRect;
	
	void OnGui() {
		//print("AttitudeRateController.OnGui");
		if (vessel.isActiveVessel) {
			Rect newWinRect = GUILayout.Window(winID, winRect, OnWindow, "ARC");
			if (newWinRect != winRect)
				defaultWinRect = winRect = newWinRect;
		}
	}
	
	void OnWindow(int id) {
		GUI.DragWindow(winDrag);
		isActive = GUILayout.Toggle(isActive, "On");
	}

}
